<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class CompanyController extends Controller
{

    public function index()
    {

        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {

            $company_data = DB::select('SELECT*FROM company_view ORDER BY id DESC');

            $currentUsername = session()->get('username');
            $logData = array('Company view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("company", compact('company_data'));
        }

    }

    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();
            $currentRowData = null;
            $btnStatus = '';
            $dataStore = '';

            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
                $dataStore =
                    '[{"tblcompanytype_id":' . '"' . $currentRowData['tblcompanytype_id'] . '",' .
                    '"tblindustry_id":' . '"' . $currentRowData['tblindustry_id'] . '"}]';
            }

            $company_type_data = DB::select('SELECT * FROM tblcompanytype ');
            $industry_data = DB::select('SELECT * FROM tblindustry');

            $currentUsername = session()->get('username');
            $logData = array('Company edition view',
                "preparing_company_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('company-edition',
                compact(
                    'btnStatus',
                    'currentRowData',
                    'dataStore',
                    'company_type_data',
                    'industry_data'
                ));

            // echo $btnStatus;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $company_name = $request->input('company_name');
                $tblcompanytype_id = $request->input('tblcompanytype_id');
                $tblindustry_id = $request->input('tblindustry_id');
                $email = $request->input('email');
                $website = $request->input('website');
                $address = $request->input('address');

                $insertion = DB::select('INSERT INTO tblcompany( name, tblcompanytype_id, tblindustry_id, email, website, address) VALUES (?,?,?,?,?,?)',
                    [$company_name, $tblcompanytype_id, $tblindustry_id, $email, $website, $address]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_company_data" => array(
                        'company_name' => $company_name,
                        'tblcompanytype_id' => $tblcompanytype_id,
                        'tblindustry_id' => $tblindustry_id,
                        'email' => $email,
                        'website' => $website,
                        'address' => $address,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('company');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! This set of company data contains duplicates.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! This set of company data contains duplicates.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('company');

                }
            }
        }
    }

    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $dataStore = $request->input('dataStore');
                $data = json_decode($dataStore, true);

                $com_id = $request->currentID;
                $company_name = $request->input('company_name');
                $tblcompanytype_id = $request->input('tblcompanytype_id') == null ? $data[0]['tblcompanytype_id'] : $request->input('tblcompanytype_id');
                $tblindustry_id = $request->input('tblindustry_id') == null ? $data[0]['tblindustry_id'] : $request->input('tblindustry_id');
                $email = $request->input('email');
                $website = $request->input('website');
                $address = $request->input('address');

                // return [$com_id, $company_name, $tblcompanytype_id, $tblindustry_id, $email, $website, $address];

                $updation = DB::select('UPDATE tblcompany SET name=?,tblcompanytype_id=?,tblindustry_id=?,email=?,website=?,address=? WHERE id=?',
                    [$company_name, $tblcompanytype_id, $tblindustry_id, $email, $website, $address, $com_id]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "updated_company_data" => array(
                        'company_name' => $company_name,
                        'tblcompanytype_id' => $tblcompanytype_id,
                        'tblindustry_id' => $tblindustry_id,
                        'email' => $email,
                        'website' => $website,
                        'address' => $address,
                        'com_id' => $com_id,
                    ),
                    "editor" => $currentUsername);

                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('company');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! This set of company data contains duplicates.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! This set of company data contains duplicates.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('company');

                }
            }
        }

    }

}
