<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class CompanyTypeController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $company_type_data = DB::select('SELECT * FROM tblcompanytype');

            $currentUsername = session()->get('username');
            $logData = array('Company type view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("company-type", compact('company_type_data'));
        }
    }

    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();

            $btnStatus = '';
            $currentRowData = null;
            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
            }

            $currentUsername = session()->get('username');
            $logData = array('Company type edition view',
                "preparing_company_type_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('company-type-edition',
                compact(
                    'currentRowData',
                    'btnStatus'));

            // echo $com_type_status;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $com_type_name = $request->input('com_type_name');
                $com_type_status = $request->input('com_type_status');

                $insertion = DB::select('INSERT INTO tblcompanytype( name, status) VALUES (?,?)',
                    [$com_type_name, $com_type_status]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_company_type_data" => array(
                        'com_type_name' => $com_type_name,
                        'com_type_status' => $com_type_status,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('company-type');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! The name of a company type must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! The name of a company type must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('company-type');

                }
            }
        }

    }

    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $currentID = $request->currentID;
                $com_type_name = $request->input('com_type_name');
                $com_type_status = $request->input('com_type_status');

                $updation = DB::select('UPDATE tblcompanytype SET name=?,status=? WHERE id=?',
                    [$com_type_name, $com_type_status, $currentID]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "updated_company_type_data" => array(
                        'com_type_name' => $com_type_name,
                        'com_type_status' => $com_type_status,
                        'tblcompanytype_id' => $currentID,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('company-type');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The name of a company type must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The name of a company type must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));
                    return redirect()->route('company-type');

                }
            }

        }
    }
}
