<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class ContactController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $contact_data = DB::select('SELECT*FROM contact_view ORDER BY id DESC');

            $currentUsername = session()->get('username');
            $logData = array('Contact view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("contact", compact('contact_data'));
        }
    }
    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();
            $currentRowData = null;
            $btnStatus = '';
            $dataStore = null;
            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
                $dataStore =
                    '[{"tblcompany_id":' . '"' . $currentRowData['tblcompany_id'] . '"}]';
            }

            $company_data = DB::select('SELECT id, name,company_type,industry_name FROM company_view');

            $currentUsername = session()->get('username');
            $logData = array('Contact edition view',
                "preparing_contact_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('contact-edition',
                compact(
                    'btnStatus',
                    'dataStore',
                    'currentRowData',
                    'company_data'));

            // echo $btnStatus;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $tblcompany_id = $request->input('tblcompany_id');
                $contact_name = $request->input('contact_name');
                $conatct_surname = $request->input('conatct_surname');
                $phone = $request->input('phone');
                $email = $request->input('email');
                $contact_status = $request->input('contact_status');
                // return [$com_id, $contact_name, $conatct_surname, $phone, $email, $contact_status];
                $insertion = DB::select('INSERT INTO tblcontact( tblcompany_id, name, surname, phone, email,status) VALUES (?,?,?,?,?,?)',
                    [$tblcompany_id, $contact_name, $conatct_surname, $phone, $email, $contact_status]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_contact_data" => array(
                        'tblcompany_id' => $tblcompany_id,
                        'contact_name' => $contact_name,
                        'conatct_surname' => $conatct_surname,
                        'phone' => $phone,
                        'email' => $email,
                        'contact_status' => $contact_status,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('contact');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! This set of contact data contains duplicates.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! This set of contact data contains duplicates.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('contact');

                }
            }
        }

    }
    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {
                $dataStore = $request->input('dataStore');
                $data = json_decode($dataStore, true);

                $contact_id = $request->currentID;
                $tblcompany_id = $request->input('tblcompany_id') == null ? $data[0]['tblcompany_id'] : $request->input('tblcompany_id');
                $contact_name = $request->input('contact_name');
                $conatct_surname = $request->input('conatct_surname');
                $phone = $request->input('phone');
                $email = $request->input('email');
                $contact_status = $request->input('contact_status');

                // return [$contact_id, $com_id, $contact_name, $phone, $contact_status];

                $updation = DB::select('UPDATE tblcontact SET tblcompany_id=?,name=?,surname=?,phone=?,email=?,status=? WHERE id=?',
                    [$tblcompany_id, $contact_name, $conatct_surname, $phone, $email, $contact_status, $contact_id]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "updated_contact_data" => array(
                        'tblcompany_id' => $tblcompany_id,
                        'contact_name' => $contact_name,
                        'conatct_surname' => $conatct_surname,
                        'phone' => $phone,
                        'email' => $email,
                        'contact_status' => $contact_status,
                        'contact_id' => $contact_id,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('contact');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! This set of contact data contains duplicates.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! This set of contact data contains duplicates.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('contact');

                }
            }
        }
    }
}
