<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class DealActivityController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $deal_activity_data = DB::select('SELECT*FROM deal_activity_view ORDER BY id DESC');

            $currentUsername = session()->get('username');
            $logData = array('Deal activity view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("deal-activity", compact('deal_activity_data'));
        }
    }
    public function prepareData(Request $request)
    {

        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {

            $d_ac_id = $request->d_ac_id;
            $btnStatus = '';
            $dataStore = null;
            $currentRowData1 = null;
            $deal_id = session()->get('deal_id');
            if ($request->btnStatus == 'add') {
                $btnStatus = 'add';
            }
            if ($request->btnStatus == 'update') {
                $btnStatus = 'update';
                $currentRowData1 = DB::select('SELECT*FROM deal_activity_view WHERE id=? ORDER BY id DESC', [$d_ac_id]);
                $dataStore = '[{"tbldeal_id ":' . '"' . $currentRowData1[0]->tbldeal_id . '"}]';

            }

            $deal_data = DB::select('SELECT * FROM tbldeal WHERE id=?', [$deal_id]);

            $currentUsername = session()->get('username');
            $logData = array('Deal activity edition view',
                "preparing_deal_activity_data" => $currentRowData1,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('deal-activity-edition',
                compact(
                    'btnStatus',
                    'currentRowData1',
                    'dataStore',
                    'deal_data'));

        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {
                $dataStore = 0;
                $btnStatus = 'update';
                $nextID = DB::select('SELECT (MAX(id)+1) as nextID FROM tbldealactivity');
                $detail = $request->input('detail');
                $deal_activity_datetime = $request->input('deal_activity_datetime');
                $newDate = Carbon::createFromFormat('Y-m-d H:i', $deal_activity_datetime)->format('Y-m-d H:i:s');
                $status = $request->input('status');
                $tbldeal_id = session()->get('deal_id');

                $insertion = DB::select('INSERT INTO tbldealactivity(id, detail, datetime, status, tbldeal_id)
                VALUES (?,?,?,?,?)',
                    [$nextID[0]->nextID == null ? 1 : $nextID[0]->nextID,
                        $detail,
                        $newDate,
                        $status,
                        $tbldeal_id]);

                $currentRowData0 = session()->get('currentRowData');
                $deal_activity_data = DB::select('SELECT*FROM deal_activity_view WHERE tbldeal_id=? ORDER BY id DESC', [$tbldeal_id]);
                $company_data = DB::select('SELECT id, name,company_type,industry_name FROM company_view');
                $contact_data = DB::select('SELECT id, CONCAT(name," ", surname) as contact_fullname,status,tblcompany_id  FROM tblcontact');
                $deal_type_data = DB::select('SELECT * FROM tbldealtype');
                $source_data = DB::select('SELECT * FROM tblsource');
                $user_data = DB::select('SELECT id, CONCAT( name," ", surname) AS user_fullname, referenceId, status FROM tbluser');
                $jsonData = '[' . $currentRowData0 . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_deal_activity_data" => array(
                        'deal_activity_id' => $nextID[0]->nextID == null ? 1 : $nextID[0]->nextID,
                        'detail' => $detail,
                        'datetime' => $newDate,
                        'status' => $status,
                        'tbldeal_id' => $tbldeal_id,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return view('deal-edition',
                    compact(
                        'btnStatus',
                        'currentRowData',
                        'dataStore',
                        'company_data',
                        'contact_data',
                        'deal_type_data',
                        'source_data',
                        'user_data',
                        'deal_activity_data'));

            } catch (\Exception $msg) {
                session()->flash('toastr', [
                    'type' => 'error',
                    'message' => 'Data inserting failed.',
                ]);
                $currentUsername = session()->get('username');
                $logData = array('Deal activity data inserting failed.', "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));
            }
        }
    }
    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {
                $dataStore = 0;
                $btnStatus = 'update';
                $deal_activity_id = $request->currentID;
                $detail = $request->input('detail');
                $deal_activity_datetime = $request->input('deal_activity_datetime');
                $newDate = Carbon::createFromFormat('Y-m-d H:i', $deal_activity_datetime)->format('Y-m-d H:i:s');
                $status = $request->input('status');

                $tbldeal_id = session()->get('deal_id');
                $currentRowData0 = session()->get('currentRowData');

                $updation = DB::select('UPDATE tbldealactivity SET
                detail=?,
                datetime=?,
                status=?,
                tbldeal_id=?
                WHERE id=?',
                    [$detail,
                        $newDate,
                        $status,
                        $tbldeal_id,
                        $deal_activity_id]);

                $deal_activity_data = DB::select('SELECT*FROM deal_activity_view WHERE tbldeal_id=? ORDER BY id DESC', [$tbldeal_id]);
                $company_data = DB::select('SELECT id, name,company_type,industry_name FROM company_view');
                $contact_data = DB::select('SELECT id, CONCAT(name," ", surname) as contact_fullname,status,tblcompany_id  FROM tblcontact');
                $deal_type_data = DB::select('SELECT * FROM tbldealtype');
                $source_data = DB::select('SELECT * FROM tblsource');
                $user_data = DB::select('SELECT id, CONCAT( name," ", surname) AS user_fullname, referenceId, status FROM tbluser');
                $jsonData = '[' . $currentRowData0 . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "updated_deal_activity_data" => array(
                        'deal_activity_id' => $deal_activity_id,
                        'detail' => $detail,
                        'datetime' => $newDate,
                        'status' => $status,
                        'tbldeal_id' => $tbldeal_id,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return view('deal-edition',
                    compact(
                        'btnStatus',
                        'currentRowData',
                        'dataStore',
                        'company_data',
                        'contact_data',
                        'deal_type_data',
                        'source_data',
                        'user_data',
                        'deal_activity_data'));

            } catch (\Exception $msg) {

                session()->flash('toastr', [
                    'type' => 'error',
                    'message' => 'Data updating failed.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Deal activity data updating failed.', "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));
            }
        }
    }

    public function closePage(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {
                $dataStore = 0;
                $btnStatus = 'update';

                $tbldeal_id = session()->get('deal_id');
                $currentRowData0 = session()->get('currentRowData');

                $deal_activity_data = DB::select('SELECT*FROM deal_activity_view WHERE tbldeal_id=? ORDER BY id DESC', [$tbldeal_id]);
                $company_data = DB::select('SELECT id, name,company_type,industry_name FROM company_view');
                $contact_data = DB::select('SELECT id, CONCAT(name," ", surname) as contact_fullname,status,tblcompany_id  FROM tblcontact');
                $deal_type_data = DB::select('SELECT * FROM tbldealtype');
                $source_data = DB::select('SELECT * FROM tblsource');
                $user_data = DB::select('SELECT id, CONCAT( name," ", surname) AS user_fullname, referenceId, status FROM tbluser');
                $jsonData = '[' . $currentRowData0 . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];

                $currentUsername = session()->get('username');
                $logData = array('Deal edition view',
                    "preparing_deal_data" => $currentRowData,
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return view('deal-edition',
                    compact(
                        'btnStatus',
                        'currentRowData',
                        'dataStore',
                        'company_data',
                        'contact_data',
                        'deal_type_data',
                        'source_data',
                        'user_data',
                        'deal_activity_data'));

            } catch (\Exception $msg) {
                session()->flash('toastr', [
                    'type' => 'error',
                    'message' => 'Data updating failed.',
                ]);
            }
        }
    }
}
