<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class DealTypeController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $deal_type_data = DB::select('SELECT * FROM tbldealtype');

            $currentUsername = session()->get('username');
            $logData = array('Deal type view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("deal-type", compact('deal_type_data'));
        }
    }

    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();

            $btnStatus = '';
            $currentRowData = null;
            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
            }

            $currentUsername = session()->get('username');
            $logData = array('Deal type edition view',
                "preparing_deal_type_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('deal-type-edition',
                compact(
                    'btnStatus',
                    'currentRowData'));

            // echo $com_type_status;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $nextID = DB::select('SELECT (MAX(id)+1) as nextID FROM tbldealtype');

                $deal_type_name = $request->input('deal_type_name');
                $deal_type_status = $request->input('deal_type_status');

                $insertion = DB::select('INSERT INTO tbldealtype(id, name, status) VALUES (?,?,?)',
                    [$nextID[0]->nextID == null ? 1 : $nextID[0]->nextID, $deal_type_name, $deal_type_status]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_deal_type_data" => array(
                        'tbldealtype_id' => $nextID[0]->nextID == null ? 1 : $nextID[0]->nextID,
                        'deal_type_name' => $deal_type_name,
                        'deal_type_status' => $deal_type_status,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('deal-type');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! The name of deal type must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! The name of deal type must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('deal-type');

                }
            }
        }
    }

    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $deal_type_id = $request->currentID;
                $deal_type_name = $request->input('deal_type_name');
                $deal_type_status = $request->input('deal_type_status');

                //echo $com_id, $company_name, $company_type, $industry, $email, $website, $address;

                $updation = DB::select('UPDATE tbldealtype SET name=?,status=? WHERE id=?',
                    [$deal_type_name, $deal_type_status, $deal_type_id]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "updated_deal_type_data" => array(
                        'deal_type_name' => $deal_type_name,
                        'deal_type_status' => $deal_type_status,
                        'tbldealtype_id' => $deal_type_id,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('deal-type');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The name of deal type must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The name of deal type must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('deal-type');

                }
            }

        }
    }
}
