<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class IndustryController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $industry_data = DB::select('SELECT * FROM tblindustry');

            $currentUsername = session()->get('username');
            $logData = array('Industry view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("industry", compact('industry_data'));
        }
    }

    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();

            $btnStatus = '';
            $currentRowData = null;
            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
            }

            $currentUsername = session()->get('username');
            $logData = array('Industry edition view',
                "preparing_industry_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('industry-edition',
                compact(
                    'currentRowData',
                    'btnStatus'));

            // echo $com_type_status;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $industry_name = $request->input('industry_name');
                $industry_status = $request->input('industry_status');

                $insertion = DB::select('INSERT INTO tblindustry( name, status) VALUES (?,?)',
                    [$industry_name, $industry_status]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_industry_data" => array(
                        'industry_name' => $industry_name,
                        'industry_status' => $industry_status,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('industry');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! The name of an industry must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! The name of an industry must be unique..', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('industry');

                }
            }
        }
    }

    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $currentID = $request->currentID;
                $industry_name = $request->input('industry_name');
                $industry_status = $request->input('industry_status');

                $updation = DB::select('UPDATE tblindustry SET name=?,status=? WHERE id=?',
                    [$industry_name, $industry_status, $currentID]);
                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "added_industry_data" => array(
                        'industry_name' => $industry_name,
                        'industry_status' => $industry_status,
                        'tblindustry_id' => $currentID,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('industry');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The name of an industry must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The name of an industry must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('industry');

                }
            }

        }
    }
}
