<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class LoginController extends Controller
{
    public function index()
    {
        session(['LoginSession' => "false"]);
        session(['showingNav' => "false"]);

        session([
            'userID' => '',
            'referentID' => '',
            'employeeName' => '',
            'employeeSurname' => '',
            'username' => '']);

        $clientIP = request()->ip();
        $logData = array('Login view', "IP" => $clientIP);
        Log::channel('sales_pipeline_log')->info(json_encode($logData));

        return view('login');
    }
    public function getLoginSession()
    {
        $sessionValue = session()->get('LoginSession');

        if (!$sessionValue | $sessionValue == "false") {
            session()->flash('toastr', [
                'type' => 'error',
                'message' => 'Please log in the system!',
            ]);
            session(['LoginSession' => "false"]);

            session([
                'userID' => '',
                'referentID' => '',
                'employeeName' => '',
                'employeeSurname' => '',
                'username' => '']);

            $logData = array('Tried to log in.');
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return redirect()->route('login');
        }

    }
    public function submit(Request $request)
    {
        $setting = (new SettingController());
        $host = $setting->getHost();

        $username = $request->input('Username');
        $password = $request->input('Password');

        $logData = array('Submited loging in.', 'username' => $username);
        Log::channel('sales_pipeline_log')->info(json_encode($logData));

        // return $this->loginWithAD($host, $username, $password);
        return $this->loginWithManualUser($username, $password);
    }
    
    public function getLoginSuccesfully()
    {
        session(['LoginSession' => "true"]);

        $this->getLoginSession();

        session()->flash('toastr', [
            'type' => 'info',
            'message' => 'Loged in successfully.',
        ]);

        $logData = array('Loged in successfully.');
        Log::channel('sales_pipeline_log')->info(json_encode($logData));

        return redirect()->route('company');
    }
    public function loginWithManualUser($username, $password)
    {
        try {
            $securityCode = (new EncryptionAndDecryptionController());
            $encryptedPassword = $securityCode->encrypt($password);

            $user = DB::select('SELECT* FROM tbluser WHERE username=? AND password=? AND status="Active"', [$username, $encryptedPassword]);

            if (count($user) == 1) {

                session([
                    'userID' => $user[0]->id,
                    'referentID' => $user[0]->referenceId,
                    'employeeName' => $user[0]->name,
                    'employeeSurname' => $user[0]->surname,
                    'username' => $user[0]->username]);
                return $this->getLoginSuccesfully();
            } else {
                session()->flash('toastr', [
                    'type' => 'error',
                    'message' => 'Username or password incorrect.',
                ]);

                $logData = array('Username or password incorrect.');
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('login');
            }
        } catch (\Exception $msg) {
            echo $msg;

        }
    }


    public function callMasterUser()
    {
        $setting = (new SettingController());
        $host = $setting->getHost();
        $masterUser = $setting->getMasterUser();
        $masterPassword = $setting->getMasterPassword();

        $securityCode = (new EncryptionAndDecryptionController());
        $username = $securityCode->decryp($masterUser);
        $password = $securityCode->decryp($masterPassword);

        return $this->loginWithAD($host, $username, $password);

    }
    
    // public function loginWithAD($host, $username, $password)
    // {
    //     if ($username && $password) {
    //         $adServer = "ldap://" . $host;
    //         $ldaprdn = $username . '@sinostech.co.th';
    //         $sessionValue = false;
    //         try {
    //             $ldap = ldap_connect($adServer);
    //             if (!$ldap) {
    //                 $response[] = ['action' => "alert", 'detail' => ['status' => 'error', 'title' => 'Error', 'detail' => 'ldap Connect Failed', 'action' => '{timeOut: 5000}']];
    //                 return response()->json($response);

    //             } else {
    //                 ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
    //                 ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);

    //                 $bind = ldap_bind($ldap, $ldaprdn, $password);

    //                 if ($bind) {

    //                     $searchBase = 'ou=SinoUsers,DC=sinostech,DC=co,DC=th';
    //                     $searchFilter1 = '(objectClass=*)';

    //                     $attributes = ['CN'];
    //                     $searchResult1 = ldap_search($ldap, $searchBase, $searchFilter1, $attributes);
    //                     if ($searchResult1) {

    //                         $data1 = ldap_get_entries($ldap, $searchResult1);
    //                         $allUsers = [];

    //                         for ($i = 0; $i < count($data1) - 1; $i++) {
    //                             if ($data1[$i]['count'] == 1) {
    //                                 $str_arr = explode(' ', $data1[$i]['cn'][0]);

    //                                 if (count($str_arr) == 2) {
    //                                     $currentUser = array(

    //                                         'employeeName' => $str_arr[0],
    //                                         'employeeSurname' => $str_arr[1],
    //                                         'referentID' => $data1[$i]['dn']);

    //                                     $allUsers[] = array('fullname' => $data1[$i]['cn'][0], $currentUser);

    //                                 }

    //                             }
    //                         }

    //                         $searchFilter2 = '(samaccountname=' . $username . ')';
    //                         $searchResult2 = ldap_search($ldap, $searchBase, $searchFilter2, $attributes);
    //                         $data2 = ldap_get_entries($ldap, $searchResult2);

    //                         $currentUsername = $data2[0]['cn'][0];
    //                         $neededDataSet = [];

    //                         for ($i = 0; $i < count($allUsers); $i++) {
    //                             $fullname = $allUsers[$i]['fullname'];
    //                             if ($fullname == $currentUsername) {
    //                                 $neededDataSet[] = $allUsers[$i];

    //                             }
    //                         }

    //                         $user_data = (new UserController());
    //                         $havingLogedIn = $user_data->index($neededDataSet);
    //                         if (count($havingLogedIn) > 0) {

    //                             session(['LoginSession' => "true"]);

    //                             $this->getLoginSuccesfully();

    //                         } else {

    //                         }

    //                     }

    //                 }
    //             }
    //         } catch (\Exception $e) {

    //             if ($e->getMessage() == "ldap_bind(): Unable to bind to server: Invalid credentials") {
    //                 session()->flash('toastr', [
    //                     'type' => 'error',
    //                     'message' => 'Invalid credentials!',
    //                 ]);
    //                 session(['LoginSession' => "false"]);
    //                 $this->getLoginSession();
    //                 return redirect()->route('login');
    //             } else {
    //                 echo $e->getMessage();
    //             }
    //         }
    //     } else {

    //     }

    // }

}
