<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;

class SourceController extends Controller
{
    public function index()
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $source_data = DB::select('SELECT * FROM tblsource');

            $currentUsername = session()->get('username');
            $logData = array('Source view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view("source", compact('source_data'));
        }
    }

    public function prepareData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            $url = $currenturl = url()->full();
            $currentRowData = null;
            $btnStatus = '';
            if (Str::contains($url, 'add')) {
                $btnStatus = 'add';
            }
            if (Str::contains($url, 'update')) {
                $btnStatus = 'update';
                $jsonData = '[' . $request->currentRowData . ']';
                $data = json_decode($jsonData, true);
                $currentRowData = $data[0];
            }

            $currentUsername = session()->get('username');
            $logData = array('Source edition view',
                "preparing_source_data" => $currentRowData,
                "editor" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('source-edition',
                compact(
                    'btnStatus',
                    'currentRowData'
                ));

            // echo $com_type_status;
        }
    }

    public function insertData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $source_name = $request->input('source_name');
                $source_status = $request->input('source_status');

                $insertion = DB::select('INSERT INTO tblsource( name, status) VALUES (?,?)',
                    [$source_name, $source_status]);

                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data inserted successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data inserted successfully.',
                    "added_source_data" => array(
                        'source_name' => $source_name,
                        'source_status' => $source_status,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('source');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data insertion failed! The name of a source must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data insertion failed! The name of a source must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('source');

                }
            }
        }
    }

    public function updateData(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {
            try {

                $source_id = $request->currentID;
                $source_name = $request->input('source_name');
                $source_status = $request->input('source_status');

                //echo $com_id, $company_name, $company_type, $industry, $email, $website, $address;

                $updation = DB::select('UPDATE tblsource SET name=?,status=? WHERE id=?',
                    [$source_name, $source_status, $source_id]);

                if (count($updation) === 1) {
                    // back()->with('success', 'Updated Successfully!');
                }
                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data has been updated.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data has been updated.',
                    "updated_source_data" => array(
                        'source_name' => $source_name,
                        'source_status' => $source_status,
                        'source_id' => $source_id,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('source');

            } catch (QueryException $exception) {

                if ($exception->getCode() === "23000") {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The name of a source must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The name of a source must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('source');

                }
            }

        }
    }
}
