<?php

namespace App\Http\Controllers;

use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Mail;

class UserController extends Controller
{
    public function index(Request $request)
    {
        $currentSession = (new LoginController())->getLoginSession();
        if ($currentSession) {
            return $currentSession;
        } else {

            $passwordChange = $request->passwordChange;

            $currentUsername = session()->get('username');
            $logData = array('Profile view', "viewer" => $currentUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return view('profile', compact('passwordChange'));
        }
    }

    public function insertWithManualUser(Request $request)
    {

        try {

            $registerName = $request->input('registerName');
            $registerSurname = $request->input('registerSurname');
            $registerUsername = $request->input('registerUsername');
            $password = $this->randomPassword();

            $securityCode = (new EncryptionAndDecryptionController());
            $encrypedPassword = $securityCode->encrypt($password);
            $encrypedRefDomainID = $securityCode->encrypt($registerName . $registerSurname);

            $insertion = DB::select('INSERT INTO tbluser( name, surname,username,password,referenceId,status) VALUES (?,?,?,?,?,?)',
                [$registerName, $registerSurname, $registerUsername, $encrypedPassword, $encrypedRefDomainID, 'Active']);

            $sender = array('senderEmail' => env('MAIL_FROM_ADDRESS', 'sales-pipeline@bounta-dev.tech'),
                'senderName' => env('MAIL_SENDER', 'Sales Pipeline Support'),
                'subject' => "User Registration for Sales Pipeline");

            $user = array('name' => $registerName, 'surname' => $registerSurname, 'username' => $registerUsername);

            Mail::send('emails', ['user' => $user, 'newPassword' => $password, 'type' => 1],
                function ($message) use ($user, $sender) {
                    $message->to($user['username'])->subject($sender['subject']);
                    $message->from($sender['senderEmail'], $sender['senderName']);
                });

            session(['currentUsername' => $registerUsername]);

            $logData = array('Data inserted successfully.',
                "added_user_data" => array(
                    'registerName' => $registerName,
                    'registerSurname' => $registerSurname,
                    'registerUsername' => $registerUsername,
                    'password' => $encrypedPassword,
                    'refDomainID' => $encrypedRefDomainID,
                ),
                "editor" => $registerUsername);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

            return $insertion;

            // return [$registerName, $registerSurname, $registerUsername, $encrypedPassword, $encrypedRefDomainID];

        } catch (QueryException $exception) {
            if ($exception->getCode() === "23000") {
                if (str_contains($exception, "fullname_user_unique") == 1) {
                    return "fullname_user_unique";
                }
                if (str_contains($exception, "username_unique") == 1) {
                    return "username_unique";
                }
            }
            dd($exception);
        }

    }

    public function updateProfile(Request $request)
    {

        try {

            $userID = $sessionValue = session()->get('userID');
            $employeeName = $request->input('profile_name');
            $employeeSurname = $request->input('profile_surname');
            $old_passowrd = $request->input('old_passowrd');
            $new_password = $request->input('new_password');
            $passwordChange = $request->input('passwordChange');

            if ($passwordChange == null || $passwordChange == "no") {

                $updation = DB::select('UPDATE tbluser SET name=?,surname=? WHERE id=?',
                    [$employeeName, $employeeSurname, $userID]);

                session([
                    'employeeName' => $employeeName,
                    'employeeSurname' => $employeeSurname]);
                session()->flash('toastr', [
                    'type' => 'success',
                    'message' => 'Data updated successfully.',
                ]);

                $currentUsername = session()->get('username');
                $logData = array('Data updated successfully.',
                    "updated_user_data" => array(
                        'employeeName' => $employeeName,
                        'employeeSurname' => $employeeSurname,
                        'userID' => $userID,
                    ),
                    "editor" => $currentUsername);
                Log::channel('sales_pipeline_log')->info(json_encode($logData));

                return redirect()->route('profile', ['passwordChange' => 'false']);

            } else {
                $securityCode = (new EncryptionAndDecryptionController());
                $encrypedPassword = $securityCode->encrypt($old_passowrd);
                $myPassword = DB::select('SELECT password FROM tbluser WHERE password=? AND id=?',
                    [$encrypedPassword, $userID]);
                if (count($myPassword) == 0) {

                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed. Incorrect password', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('profile', ['passwordChange' => 'true']);
                } else {
                    $securityCode = (new EncryptionAndDecryptionController());
                    $encrypedPassword = $securityCode->encrypt($new_password);
                    $updation = DB::select('UPDATE tbluser SET name=?,surname=?,password=? WHERE id=?',
                        [$employeeName, $employeeSurname, $encrypedPassword, $userID]);
                    session([
                        'employeeName' => $employeeName,
                        'employeeSurname' => $employeeSurname]);
                    session()->flash('toastr', [
                        'type' => 'success',
                        'message' => 'Data updated successfully.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updated successfully.',
                        "updated_user_data" => array(
                            'employeeName' => $employeeName,
                            'employeeSurname' => $employeeSurname,
                            'encrypedPassword' => $encrypedPassword,
                            'userID' => $userID,
                        ),
                        "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                    return redirect()->route('profile', ['passwordChange' => 'false']);

                }
            }
        } catch (QueryException $exception) {
            if ($exception->getCode() === "23000") {
                if (str_contains($exception, "fullname_user_unique") == 1) {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The name and surname contain duplicates.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The name and surname contain duplicates.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                }
                if (str_contains($exception, "username_unique") == 1) {
                    session()->flash('toastr', [
                        'type' => 'error',
                        'message' => 'Data updation failed! The username must be unique.',
                    ]);

                    $currentUsername = session()->get('username');
                    $logData = array('Data updation failed! The username must be unique.', "editor" => $currentUsername);
                    Log::channel('sales_pipeline_log')->info(json_encode($logData));

                }

                return redirect()->route('profile', ['passwordChange' => 'false']);
            }
        }

    }

    public function forgotPassword()
    {

        $clientIP = request()->ip();
        $logData = array(
            'Forgot password view',
            "reset_by_ip" => $clientIP);
        Log::channel('sales_pipeline_log')->info(json_encode($logData));

        return view('forgot-password');
    }

    public function resetPassword(Request $request)
    {
        $username = $request->input('username');
        $users = DB::select('SELECT * FROM tbluser WHERE username=? AND status="Active"', [$username]);
        if (count($users) == 0) {
            session()->flash('toastr', [
                'type' => 'error',
                'message' => $username . ' is not found!',
            ]);
            return redirect()->route('forgot-password');
        }

        $user = $users[0];
        $newPassword = $this->randomPassword();
        $sender = array('senderEmail' => env('MAIL_FROM_ADDRESS', 'sales-pipeline@bounta-dev.tech'),
            'senderName' => env('MAIL_SENDER', 'Sales Pipeline Support'),
            'subject' => 'Password resetting for Sales Pipeline');

        Mail::send('emails', ['user' => $user, 'newPassword' => $newPassword, 'type' => 2],
            function ($message) use ($user, $sender) {
                $message->to($user->username)->subject($sender['subject']);
                $message->from($sender['senderEmail'], $sender['senderName']);
            });

        // $name = "Funny Coder";
        // Mail::to($user->username)->send(new SalesPipelineSupportMail($name));

        $rs = $this->updatePassword($user, $newPassword);
        return redirect()->route('successful-result', ['type' => 2]);

    }

    public function randomPassword()
    {
        $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $pass = array();
        $alphaLength = strlen($alphabet) - 1;
        for ($i = 0; $i < 8; $i++) {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }
        return implode($pass);
    }

    public function updatePassword($user, $newPassword)
    {

        $securityCode = (new EncryptionAndDecryptionController());
        $encrypedPassword = $securityCode->encrypt($newPassword);

        try {

            $updation = DB::select('UPDATE tbluser SET password=? WHERE username=? AND status="Active"',
                [$encrypedPassword, $user->username]);

            session(['currentUsername' => $user->username]);

            $clientIP = request()->ip();
            $logData = array(
                'User: ' . $user->username . ' has been reset password to [' . $encrypedPassword . ']',
                "reset_by_ip" => $clientIP);
            Log::channel('sales_pipeline_log')->info(json_encode($logData));

        } catch (\Exception $msg) {

        }

    }

    public function successfulResult(Request $request)
    {
        $type = $request->type;
        session([
            'LoginSession' => 'false',
            'userID' => '',
            'referentID' => '',
            'employeeName' => '',
            'employeeSurname' => '',
            'username' => '']);

        return view('successful-result', compact('type'));

    }
    // public function useAD($neededDataSet)
    // {
    //     $referenceId = $neededDataSet[0][0]['referentID'];
    //     $securityCode = (new EncryptionAndDecryptionController());
    //     $encrypedRefDomainID = $securityCode->encrypt($referenceId);
    //     $user_data = DB::select('SELECT * FROM tbluser WHERE referenceId=?', [$encrypedRefDomainID]);
    //     if (count($user_data) > 0) {
    //         return $user_data;
    //     } else {
    //         $record = $this->insertWithAD($neededDataSet);
    //         if ($record > 0) {

    //             $user_data = DB::select('SELECT * FROM tbluser WHERE referenceId=?', [$encrypedRefDomainID]);
    //         }
    //         return $user_data;

    //     }
    // }

    // public function insertWithAD($neededDataSet)
    // {

    //     try {

    //         $employeeName = $neededDataSet[0][0]['employeeName'];
    //         $employeeSurname = $neededDataSet[0][0]['employeeSurname'];
    //         $referenceId = $neededDataSet[0][0]['referentID'];
    //         // $username = $neededDataSet[0][0]['referentID'];

    //         $securityCode = (new EncryptionAndDecryptionController());
    //         $encrypedRefDomainID = $securityCode->encrypt($referenceId);

    //         $insertion = DB::select('INSERT INTO tbluser( name, surname,referenceId,status) VALUES (?,?,?,?)',
    //             [$employeeName, $employeeSurname, $encrypedRefDomainID, 'Active']);

    //         session([
    //             'userID' => '',
    //             'referentID' => $encrypedRefDomainID,
    //             'employeeName' => $employeeName,
    //             'employeeSurname' => $employeeSurname,
    //             'username' => '']);

    //         return $insertion;

    //     } catch (\Exception $msg) {
    //         echo 'Message: ' . $msg->getMessage();

    //     }

    // }
}
